@@ -1,12 +1,13 @@
 from django import forms
-from django.contrib.auth.hashers import make_password
+from django.contrib.auth.hashers import check_password
 from django.utils.translation import ugettext as _
 
 from is_core.forms.forms import SmartForm
 
 from auth_token.config import settings
 from auth_token.contrib.common.forms import AuthenticationCleanMixin, TokenAuthenticationMixin
-from auth_token.models import Token
+from auth_token.models import AuthorizationToken, hash_key
+from auth_token.utils import get_valid_otp, check_authorization_request
 
 
 class TokenAuthenticationSmartForm(TokenAuthenticationMixin, AuthenticationCleanMixin, SmartForm):
@@ -22,18 +23,23 @@ class LoginCodeVerificationForm(SmartForm):
         super().__init__(*args, **kwargs)
 
     def _init_code(self, field):
-        if settings.TWO_FACTOR_DEBUG_TOKEN_SMS_CODE:
-            field.initial = settings.TWO_FACTOR_DEBUG_TOKEN_SMS_CODE
+        if settings.AUTHORIZATION_REQUEST_OTP_DEBUG_CODE:
+            field.initial = settings.AUTHORIZATION_REQUEST_OTP_DEBUG_CODE
+
+    def get_authorization_request(self):
+        return self.request.token.authorization_requests.filter(
+            slug=settings.TWO_FACTOR_AUTHORIZATION_SLUG
+        ).first('-created_at')
 
     def get_user(self):
         return self.request.token.user
 
     def clean_code(self):
         code = self.cleaned_data.get('code')
-        if (make_password(code, salt=Token.TWO_FACTOR_CODE_SALT) != self.request.token.two_factor_code
-                and (not settings.TWO_FACTOR_DEBUG_TOKEN_SMS_CODE or settings.TWO_FACTOR_DEBUG_TOKEN_SMS_CODE != code)):
-            raise forms.ValidationError(_('The inserted value does not correspond to the sent code.'))
+        authorization_requests = self.get_authorization_request()
+
+        if authorization_requests and check_authorization_request(authorization_requests, otp_secret_key=code):
+            self.request.token.change_and_save(is_authenticated=True, update_only_changed_fields=True)
         else:
-            self.request.token.is_authenticated = True
-            self.request.token.save()
+            raise forms.ValidationError(_('The inserted value does not correspond to the sent code.'))
         return code
